program Interfaces;
{%DelphiDotNetAssemblyCompiler '$(SystemRoot)\microsoft.net\framework\v1.1.4322\system.drawing.dll'}

uses
  SysUtils, StdCtrls;

{.$DEFINE INTERFACE}
{$APPTYPE CONSOLE}

{$IFDEF CLR}
{%DelphiDotNetAssemblyCompiler '$(SystemRoot)\microsoft.net\framework\v1.1.4322\system.drawing.dll'}
{$ENDIF}

type
  IString = interface
  ['{B20CF5C0-4042-11D4-B84D-444553540000}']
    function GetText: String;
    procedure SetText(NewText: String);
    property Text: String read GetText write SetText;
  end;

  IEncryption = interface(IString)
    procedure EnCrypt;
    procedure DeCrypt;
  end;

  IPrint = interface
    procedure PrintText;
    function GetText: String;
  end;

type
  TSecureMessage = class(TEdit, IEncryption, IString, IPrint)
    function IEncryption.GetText = EncryptionGetText;
  {$IFDEF WIN32}
    function IString.GetText = EncryptionGetText;
  {$ENDIF}
    function IPrint.GetText = PrintGetText;
  private
    FText: String;
  public
    procedure EnCrypt;
    procedure DeCrypt;
  public
    function EncryptionGetText: String;
    procedure SetText(NewText: String);
    procedure PrintText;
    function PrintGetText: String;
  published
    property Text: String read EncryptionGetText write SetText;
  end;

type
  TMyEdit = class(TEdit, IString)
  private
    FText: String;
  public
    function GetText: String;
    procedure SetText(NewText: String);
    property Text: String read GetText write SetText;
  end;

procedure TSecureMessage.EnCrypt;
var
  i: Integer;
begin
  for i:=1 to Length(FText) do FText[i] := Chr(Ord(FText[i])+1);
end;

procedure TSecureMessage.DeCrypt;
var
  i: Integer;
begin
  for i:=1 to Length(FText) do FText[i] := Chr(Ord(FText[i])-1);
end;

function TSecureMessage.EncryptionGetText: String;
begin
  Result := FText
end;

procedure TSecureMessage.SetText(NewText: String);
begin
  FText := NewText
end;

function TSecureMessage.PrintGetText: String;
begin
  Result := 'Print: ' + FText
end;

procedure TSecureMessage.PrintText;
begin
  writeln(FText);
end;


function Count(const Str: IString): Integer;
begin
  Result := Length(Str.GetText)
end;

var
{$IFDEF INTERFACE}
  SecureMessage: IEncryption;
{$ELSE}
  SecureMessage: TSecureMessage;
{$ENDIF}
  X: String;

{ TMyEdit }

function TMyEdit.GetText: String;
begin
  Result := FText
end;

procedure TMyEdit.SetText(NewText: String);
begin
  FText := NewText;
end;

begin
  SecureMessage := TSecureMessage.Create(nil);
  SecureMessage.Text := 'hello';
  writeln(Count(SecureMessage));
  SecureMessage.Encrypt;

{$IFNDEF INTERFACE}
  SecureMessage.PrintText;
{$ENDIF}

  SecureMessage.Decrypt;

{$IFNDEF INTERFACE}
  SecureMessage.PrintText;
{$ENDIF}

{$IFDEF WIN32}
  try
    X := (SecureMessage as IString).Text;
    writeln('SecureMessage (directly) implements IString');
  except
    on E: Exception do
      writeln(E.Message);
  end;
{$ELSE}
  if (SecureMessage is IString) then
    writeln('SecureMessage (directly) implements IString');
{$ENDIF}
  writeln(SecureMessage.Text);

{$IFNDEF INTERFACE}
  {$IFDEF WIN32}
    write('Hit <enter> to crash...');
    readln;
  {$ENDIF}
  SecureMessage.Free; // crashes in Win32...
  // Note: only crashes if we use the "as" operator above!!
{$ENDIF}
  readln
end.
