unit Oblong.RightEdit;

interface

uses
  SysUtils, Classes, Controls, StdCtrls;

type
  TMaxLengthEvent = procedure(Sender: TObject; Length: Integer) of object;

  TRightEdit = class(TCustomMemo)
  private
    FOnMaxLength: TMaxLengthEvent;
    { Private declarations }
  protected
    procedure Change; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    { Protected declarations }
  public
    constructor Create(AOwner: TComponent); override;
    procedure SetBounds(ALeft: Integer; ATop: Integer; AWidth: Integer;
      AHeight: Integer); override;
  published
    property OnChange;
    property OnMaxLength: TMaxLengthEvent read FOnMaxLength write FOnMaxLength;
  end;

implementation

{ TRightEdit }

procedure TRightEdit.Change;
begin
  inherited;
  if (MaxLength > 0) and (Length(Text) >= MaxLength) then
    if Assigned(FOnMaxLength) then
      FOnMaxLength(Self, MaxLength)
end;

constructor TRightEdit.Create(AOwner: TComponent);
begin
  inherited;
  Alignment := taRightJustify
end;

procedure TRightEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if Key in [10, 13] then
    Key := 0
  else
    inherited
end;

procedure TRightEdit.KeyPress(var Key: Char);
begin
  if Key in [#10, #13] then
    Key := #0
  else
    inherited;
end;

procedure TRightEdit.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  if AHeight > 2 * Abs(Font.Height) then
    AHeight := 2 * Abs(Font.Height);
  inherited;
end;

end.
