unit eBob42.DBDateTimePicker;
interface
uses
  SysUtils, Classes, Controls, ComCtrls,
  DB, DBCtrls;

type
  TDBDateTimePicker = class(TDateTimePicker)
  private
    { Private declarations }
    FFieldDataLink: TFieldDataLink;
    function GetDataField: String;
    function GetDataSource: TDataSource;
    procedure SetDataField(const Value: String);
    procedure SetDataSource(const Value: TDataSource);
  protected
    procedure Change; override;
    { Protected declarations }
    procedure DataChange(Sender: TObject);
  public
    destructor Destroy; override;
    constructor Create(AOwner: TComponent); override;
    { Public declarations }
  published
    { Published declarations }
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DateField: String read GetDataField write SetDataField;
  end;

implementation

{ TDBDateTimePicker }

procedure TDBDateTimePicker.Change;
begin
  inherited;
  Enabled := Assigned(FFieldDataLink);
  if Enabled then
    if Assigned(FFieldDataLink.Field) then
    begin
      if not (FFieldDataLink.Field.DataSet.State in [dsEdit, dsInsert]) then
        FFIeldDataLink.Field.DataSet.Edit;
      FFieldDataLink.Field.AsDateTime := DateTime
    end
end;

constructor TDBDateTimePicker.Create(AOwner: TComponent);
begin
  inherited;
  FFieldDataLink := TFieldDataLink.Create;
  FFieldDataLink.OnDataChange := DataChange
end;

procedure TDBDateTimePicker.DataChange(Sender: TObject);
begin
  Enabled := Assigned(FFieldDataLink);
  if Enabled then
    if Assigned(FFieldDataLink.Field) then
    try
      Date := FFieldDataLink.Field.AsDateTime
    except
      on E: Exception do
      begin
        Hint := E.Message;
        Date := SysUtils.Date-1
      end
    end
    else Date := 0
  else Date := 0
end;

destructor TDBDateTimePicker.Destroy;
begin
  FFieldDataLink.Free;
  inherited;
end;

function TDBDateTimePicker.GetDataField: String;
begin
  Result := FFieldDataLink.FieldName
end;

function TDBDateTimePicker.GetDataSource: TDataSource;
begin
  Result := FFieldDataLink.DataSource
end;

procedure TDBDateTimePicker.SetDataField(const Value: String);
begin
  FFieldDataLink.FieldName := Value
end;

procedure TDBDateTimePicker.SetDataSource(const Value: TDataSource);
begin
  FFieldDataLink.DataSource := Value
end;

end.
