unit eBob42.RightEdit;
interface
uses
  SysUtils, Classes, Controls, StdCtrls;

type
  TMaxLengthEvent = procedure(Sender: TObject; Length: Integer) of object;

  TRightEdit = class(TCustomMemo)
  private
    FOnMaxLength: TMaxLengthEvent;
    { Private declarations }
  protected
    procedure Change; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    { Protected declarations }
  public
    constructor Create(AOwner: TComponent); override;
    procedure SetBounds(ALeft: Integer; ATop: Integer; AWidth: Integer;
      AHeight: Integer); override;
    { Public declarations }
  published
    property Text;
    { Published declarations - copied from TEdit
    property Anchors;
    property AutoSelect;
    property AutoSize;
    property BevelEdges;
    property BevelInner;
    property BevelKind default bkNone;
    property BevelOuter;
    property BevelWidth;
    property BiDiMode;
    property BorderStyle;
    property CharCase;
    property Color;
    property Constraints;
    property Ctl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    property ImeMode;
    property ImeName;
    property MaxLength;
    property OEMConvert;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseActivate;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag; }
    property OnMaxLength: TMaxLengthEvent read FOnMaxLength write FOnMaxLength;
  end;

implementation

{ TRightEdit }

procedure TRightEdit.Change;
var
  MyText: String;
  CrPos: integer;
begin
  MyText := Text;
  CrPos := Pos(#13, MyText);
  if CrPos > 0 then Text := Copy(MyText, 1, CrPos-1)
  else
    inherited;
  // now check for max length...
  if (MaxLength > 0) and (Length(Text) >= MaxLength) then
    if Assigned(FOnMaxLength) then FOnMaxLength(Self,MaxLength)
end;

constructor TRightEdit.Create(AOwner: TComponent);
begin
  inherited;
  Alignment := taRightJustify;
  Align := alNone;
  ScrollBars := ssNone;
  WantReturns := False;
  WantTabs := False;
  WordWrap := False
end;

procedure TRightEdit.KeyPress(var Key: Char);
// prevent Ctrl+Enter as well
begin
  if Key in [#10, #13] then Key := #0
  else
    inherited
end;

procedure TRightEdit.KeyDown(var Key: Word; Shift: TShiftState);
// prevent Ctrl+Enter as well
begin
  if Key in [10, 13] then Key := 0
  else
    inherited
end;

procedure TRightEdit.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  if AHeight > (2 * abs(Font.Height)) then AHeight := 2 * abs(Font.Height);
  inherited
end;

end.
