unit OblongKelvinProperty;

interface

uses
  DesignIntf, DesignEditors, VCLEditors, Classes, Types, Graphics;

type
  TKelvinProperty = class(TFloatProperty, ICustomPropertyListDrawing)
  public
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
    procedure ListDrawValue(const Value: string; ACanvas: TCanvas;
      const ARect: TRect; ASelected: Boolean);
    procedure ListMeasureHeight(const Value: string; ACanvas: TCanvas;
      var AHeight: Integer);
    procedure ListMeasureWidth(const Value: string; ACanvas: TCanvas;
      var AWidth: Integer);
  public
    function GetAttributes: TPropertyAttributes; override;
  end;

implementation

{ TKelvinProperty }

function TKelvinProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paMultiSelect, paValueList]
end;

const
  Values: array[1..4] of String =
    ( 'Absolute zero', 'Freezing', 'Room temperature', 'Boiling' );
  Temps: array[1..4] of Double = (0, 273.15, 297.15, 373.15);

procedure TKelvinProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
begin
  for I := Low(Values) to High(Values) do
    Proc(Values[I])
end;

procedure TKelvinProperty.ListMeasureHeight(const Value: string;
  ACanvas: TCanvas; var AHeight: Integer);
begin
  //
end;

procedure TKelvinProperty.ListDrawValue(const Value: string; ACanvas: TCanvas;
  const ARect: TRect; ASelected: Boolean);
const
  Colours: array[1..4] of TColor = (clNavy, clBlue, clLime, clRed);
var
  I: Integer;
begin
  for I := Low(Values) to High(Values) do
    if Value = Values[I] then
    begin
      ACanvas.Brush.Color := Colours[I];
      ACanvas.Font.Color := clWhite;
      Break;
    end;
  DefaultPropertyListDrawValue(Value, ACanvas, ARect, ASelected);
end;

procedure TKelvinProperty.ListMeasureWidth(const Value: string;
  ACanvas: TCanvas; var AWidth: Integer);
begin
  //
end;

procedure TKelvinProperty.SetValue(const Value: string);
var
  I: Integer;
begin
  for I := Low(Values) to High(Values) do
  begin
    if Value = Values[I] then
    begin
      SetFloatValue(Temps[I]);
      Exit
    end;
  end;
  //Wasn't a special value
  inherited;
end;

end.
